# Example Problems

## About

This folder contains example problems for MAiNGO. 
If you are new to MAiNGO, we recommend to look at the following documents in this order:
* `doc/html/index.html`: This is the manual for MAiNGO. It includes information on how to install and execute MAiNGO.
* `01_BasicExample`: This folder contains the most basic example for solving problems with MAiNGO.

Most examples are available in three forms:
* as text-based input using the Syntax of the [ALE library](https://git.rwth-aachen.de/avt.svt/public/libale)
* as C++ problem file for the C++ API of MAiNGO
* as Python script using the Python API of MAiNGO.

To solve a problem via the text-based ALE input, you need to build MAiNGO with the CMake variable `MAiNGO_build_parser` set to `true`.
After compiling MAiNGO, you obtain an executable called `MAiNGO` which contains a parser for reading ALE input.
By default, MAiNGO searches for a problem file called `problem.txt` in the current working directory from which you executed MAiNGO.
You can specify a different problem file name (including an absolute or relative path) as a command line argument.
As an optional second command line argument, you can specify the file name (and path) of a settings file. If no settings file is specified, MAiNGO looks for a file called `MAiNGOSettings.txt`.
An example for a settings file can be found in this directory (`examples`).

To solve a problem via the C++ API of MAiNGO, you need to build MAiNGO with the CMake variable `MAiNGO_build_standalone` set to `true`.
The problem is implemented by specializing the abstract `MAiNGOModel` class. In the examples, this is accomplished in a dedicated header file (e.g., `01_BasicExample/problem.h`).
You need to include the header file with your problem definition in the `mainCppApi.cpp` file in this directory and then compile MAiNGO. This will produce an executable called `MAiNGOcpp`.
When running this executable, the problem will be solved. Here, the only (optional) command line argument is the settings file name.

To solve a problem via the Python API of MAiNGO, you need to build MAiNGO with its Python interface by setting the CMake variable `MAiNGO_build_python_interface` to `true`,
which generates a Python package called `pymaingo`. You can then use MAiNGO from Python as shown, e.g., in the file `01_BasicExample\examplePythonInterface.py`, which you can run via `python3 examplePythonInterface.py`.
For this to work, you need to ensure that the `pymaingo` package is available when executing the above command.
For example, you can copy the `examplePythonInterface.py` to your build directory, which (after a successful build) should contain a sub-directory called `pymaingo`, and execute the command in your build directory.